/*
 * Devemos enviar para GPU arranjos com dados de vértices (VBO's). Estes vértices
 * 
 */


 /**
   Funcão que pego o GET
 * Created by: http://gustavopaes.net
 * Created on: Nov/2009
 */
function _GET(name)
{
  var url   = window.location.search.replace("?", "");
  var itens = url.split("&");

  for(n in itens)
  {
    if( itens[n].match(name) )
    {
      return decodeURIComponent(itens[n].replace(name+"=", ""));
    }
  }
  return null;
}

 /**
  FIM Funcão que pego o GET ***/

var gl = null; //opengl context

var canvas = null; // the canvas
var bkgColor = null; // bgk/clear color
var model = null; // current loaded model

var context = "webgl";

//buffer com os vertices.
var trianglesVerticeBuffer = null;
//indices
var indexBuffer = null;
//cores
var colorBuffer = null;
//normais
var normalBuffer = null;

//usado para passar os atributos para o shader
var vertexPositionAttribute = null;
var vertexColorAttribute   = null;
var normalAttribute         =null;
var glProgram = null;

var wireframe = true;

var PASS_TRU_SHADER = 0, FLAT_SHADER = 1, PHONG_SHADER = 2;

var currentShader = PASS_TRU_SHADER;

//a ideia de camera eh feita utilizando uma camera virtual. eh oferecida 
//as transformacoes ocorrem: o vértice do espcaço do objeto é transformado
//para o espaco do mundo. Este entao é transportado para o espaço da camera
//https://www.opengl.org/archives/resources/faq/technical/viewing.htm
//mathematics for computer graphics, John Vince
var pMatrix = mat4.create();

//http://www.opengl-tutorial.org/beginners-tutorials/tutorial-3-matrices/
var mMatrix = mat4.create();

//matriz da camera
var vMatrix = mat4.create();



//matriz de projetcao de normais
var normalMatrix = mat3.create();


var numVtx= 0;
//inicializa as configs do webgl, e entra no loop de desenho.
function start()
{
    //model    = new MeshData("../meshes/homer.off");
    arquivo = _GET("malha");
    shader = _GET("shader");
    model    = new MeshData("arquivos/"+arquivo);
    var i;
	//seta a cor dos vertices
    for(i = 0; i < model.vertexNumber; i++){
        
        model.vertices[i].color = new Color(1, 1, 0, 1);
    }
    bkgColor = new Color(0.9,0.9,0.9,1.0);
    canvas = document.getElementById("viewmodel");
    
    setCoordinates(canvas);
    
    mat4.identity(pMatrix);
    mat4.perspective(pMatrix, 45, canvas.width / canvas.height, 0.1, 100.0);
    
    mat4.identity(vMatrix);
    mat4.lookAt(vMatrix,[0,0,2], [0,0,0],[0,1,0]);
    
    mat4.identity(mMatrix, mMatrix);
    
    mat3.normalFromMat4(normalMatrix, mMatrix);
    
    setupWebGL();
    if (shader == "phong")
        setShader(PHONG_SHADER);
    else if (shader == "flat")
        setShader(FLAT_SHADER);
    else
        setShader(PASS_TRU_SHADER); 
    setupWebGL();
    getMatrixUniforms();
    setupBuffers();
    
    
    
    /*
    setupWebGL();    
    setShader(FLAT_SHADER);
    getMatrixUniforms();
    setupBuffers();
    draw();
    */
   
   //loop de animacao
    (function animLoop(){
        setupWebGL();
        getMatrixUniforms();
        draw();
        window.requestAnimationFrame(animLoop, canvas);
    })();
    
}

//set configs
function setupWebGL()
{
    //mat3.transpose(normalMatrix, normalMatrix);
    try{
        gl = canvas.getContext(context);
    }catch(e){
    }
    if(gl)
    {
        //background clear color
        gl.clearColor(bkgColor.r, bkgColor.g,   bkgColor.b, bkgColor.a);
        gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);
        gl.enable(gl.DEPTH_TEST);
        gl.viewport(0, 0, canvas.width, canvas.height);
    }
    else
    {
        alert( "Error: Your browser does not appear to support WebGL.");
    }
}



//Desenha a cena
function draw()
{
    /*
     * vértices possuem atributos que podem ser passados para o shader. 
     * Aqui, descobre onde o atributo está (aVertexPosition), e binda o buffer de
     *vértices (trianglesVerticeBuffer). Após isso, diz que o buffer será distribuido
     *para os atributos vertexPositionAttribute, de 3 em 3 (no buffer), o tipo (float), 
     *nao normalizados, 
     *  
     */
    
    setMatrixUniforms();
    
    vertexPositionAttribute = gl.getAttribLocation(glProgram, "aVertexPosition");
    gl.enableVertexAttribArray(vertexPositionAttribute);
    gl.bindBuffer(gl.ARRAY_BUFFER, trianglesVerticeBuffer);
    //explica como os dados funcionam
    gl.vertexAttribPointer(vertexPositionAttribute, 3, gl.FLOAT, false, 0, 0);
    
    
   
    if(currentShader != PASS_TRU_SHADER)
    {
        
        normalAttribute = gl.getAttribLocation(glProgram, "aVertexNormal");
       // document.getElementById("debug").innerHTML+= "Normal loc: " + normalAttribute + "<br>";
        gl.enableVertexAttribArray(normalAttribute);
        gl.bindBuffer(gl.ARRAY_BUFFER, normalBuffer);
        gl.vertexAttribPointer(normalAttribute, 3, gl.FLOAT, false, 0, 0);
    }
    
    
    //cor dos vertices
    vertexColorAttribute = gl.getAttribLocation(glProgram, "aVertexColor");
    
    gl.enableVertexAttribArray(vertexColorAttribute);
    gl.bindBuffer(gl.ARRAY_BUFFER, colorBuffer);
    gl.vertexAttribPointer(vertexColorAttribute, 3, gl.FLOAT, false, 0, 0);
    
    //triangulos, primeiro vértice, núm de vértices
    if(currentShader == FLAT_SHADER)
    {
        gl.drawArrays(gl.TRIANGLES, 0, numVtx);
        
    }
    else{
        gl.drawElements(gl.TRIANGLES, indexBuffer.number_vertex_points, gl.UNSIGNED_SHORT, 0);
    }
}


// eh necessario criar o Vertex Buffers Object do modelo carregado
function setupBuffers()
{
    var triangleVertices    = null; 
    var indices             = null;
    var color               = null; 
  
    if(currentShader == FLAT_SHADER)
    {
        triangleVertices = model.createPerFaceVertexBuffer();
        color            = model.createPerFaceColorBuffer();
        var normals = model.createPerFaceNormalBuffer();
        normalBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, normalBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(normals), gl.STATIC_DRAW);
       
        numVtx = triangleVertices.length / 3;

    }
    else //if(currentShader == PASS_TRU_SHADER)
    {
        triangleVertices = model.createVertexBuffer();
        indices          = model.createVertexIndex();
        color            = model.createColorBuffer();
        numVtx = indices.length ;
        indexBuffer = gl.createBuffer();
        indexBuffer.number_vertex_points = indices.length;
        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, indexBuffer);
        gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, new Uint16Array(indices),gl.STATIC_DRAW);
        
        if(currentShader == PHONG_SHADER){
            var normals = model.createNormalBuffer();
            normalBuffer = gl.createBuffer();
            gl.bindBuffer(gl.ARRAY_BUFFER, normalBuffer);
            gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(normals), gl.STATIC_DRAW);
        }
    }
    
  
    trianglesVerticeBuffer = gl.createBuffer();
    gl.bindBuffer(gl.ARRAY_BUFFER, trianglesVerticeBuffer);
    gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(triangleVertices), gl.STATIC_DRAW);
    
    colorBuffer = gl.createBuffer();
    gl.bindBuffer(gl.ARRAY_BUFFER, colorBuffer);
    gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(color), gl.STATIC_DRAW);
}


function setShader(shader)
{
    switch (shader){
        case PASS_TRU_SHADER:
        {
            currentShader = PASS_TRU_SHADER;
            initShaders(passTruVtx,passTruFrg);
            break;
        }
        case FLAT_SHADER:
        {
            currentShader = FLAT_SHADER;
            initShaders(flatVtx,flatFrag);
            break;
        }
        case PHONG_SHADER:
        {
            currentShader = PHONG_SHADER;
            initShaders(phongVtx,phongFrag);
            break;
        }
    }
}

//funcoes para enviar as matrizes para o shader
function getMatrixUniforms(){
    glProgram.pMatrixUniform = gl.getUniformLocation(glProgram, "uPMatrix");
    glProgram.mMatrixUniform = gl.getUniformLocation(glProgram, "uMMatrix");
    glProgram.normalMatrixUniform = gl.getUniformLocation(glProgram, "uNormalMatrix");
    glProgram.vMatrixUniform = gl.getUniformLocation(glProgram, "uVMatrix");
}
function setMatrixUniforms() {
    gl.uniformMatrix4fv(glProgram.pMatrixUniform, false, pMatrix);
    gl.uniformMatrix4fv(glProgram.mMatrixUniform, false, mMatrix);
    gl.uniformMatrix3fv(glProgram.normalMatrixUniform, false, normalMatrix);
    gl.uniformMatrix4fv(glProgram.vMatrixUniform, false, vMatrix);
}

//load, compile, init shaders
//carrega o shader de vértice e o de fragmento
function initShaders(vertex, fragment)
{
    if( (vertex !=null) && (fragment !=null)){
        //compile shaders
        vertexShader = makeShader(vertex, gl.VERTEX_SHADER,"vertex");
        fragmentShader = makeShader(fragment, gl.FRAGMENT_SHADER,"Fragment");
        //create program
        glProgram = gl.createProgram();
        //attach and link shaders to the program
        gl.attachShader(glProgram, vertexShader);
        gl.attachShader(glProgram, fragmentShader);
        gl.linkProgram(glProgram);
        if (!gl.getProgramParameter(glProgram, gl.LINK_STATUS)) {
        alert("Unable to initialize the shader program.");
        }
        //use program
        gl.useProgram(glProgram);
    }
}



//ass seen in Danchilla
//cria um programa shader.
function makeShader(src, type, what)
{
    //compile the vertex shader
    var shader = gl.createShader(type);
    gl.shaderSource(shader, src);
    gl.compileShader(shader);
    if (!gl.getShaderParameter(shader, gl.COMPILE_STATUS)) {
        alert("Error compiling shader "+what+": "+" "+ gl.getShaderInfoLog(shader));
    }
    return shader;
}


function Color (r, g, b, a){
    this.r = r;
    this.g = g;
    this.b = b;
    this.a = a;
}

function setBackgroundColor(color)
{
    bkgColor = color;
}

/**
    References:
    Danchilla, Brian. Beginning WebGL for HTML5
 */

